package com.muyu.rule.server.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.enums.SystemYesNo;
import com.muyu.rule.common.domain.DataValue;
import com.muyu.rule.common.domain.RuleEngineVersion;
import com.muyu.rule.common.utils.OssUtil;
import com.muyu.rule.server.EngineConfig;
import com.muyu.rule.server.basic.BasicEngine;
import com.muyu.rule.server.load.ExternalClassLoader;
import com.muyu.rule.server.mapper.RuleEngineVersionMapper;
import com.muyu.rule.server.service.RuleEngineVersionService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;
import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import static com.muyu.rule.server.constant.SuffixClass.Suffix_CLASS;
import static com.muyu.rule.server.constant.SuffixClass.Suffix_JAVA;

/**
 * @Author：张承志
 * @Package：com.muyu.rule.server.service.impl
 * @Project：cloud-etl-rule
 * @name：RuleEngineServiceImpl
 * @Date：2024/8/25 14:17
 */
@Log4j2
@Service
public class RuleEngineVersionServiceImpl extends ServiceImpl<RuleEngineVersionMapper, RuleEngineVersion> implements RuleEngineVersionService {

    @Override
    public List<RuleEngineVersion> selectRuleEngineVersion(Long id) {

        LambdaQueryWrapper<RuleEngineVersion> queryWrapper = new LambdaQueryWrapper<>();

        queryWrapper.eq(
                RuleEngineVersion::getRuleId, id);

        List<RuleEngineVersion> versionList = this.list(queryWrapper);

        return versionList;
    }

    @Override
    public void activate(Long id) {
        this.updateOpen(id, SystemYesNo.YES.getCode());
    }

    @Override
    public void disable(Long id) {
        this.updateOpen(id, SystemYesNo.NO.getCode());
    }



    /**
     * bucket名称
     *
     * @return
     */
    private static String bucketName = "zcz-vfd-1000";

    /**
     * 上传
     * @param versionClazz java源代码
     * @param className    类的名
     */
    @Override
    public void Upload(String versionClazz, String className) {

        /**
         * 将java源码存入OSS
         */
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(versionClazz.getBytes());
        OssUtil.uploadFileInputStreamForBucket(bucketName, className + Suffix_JAVA, byteArrayInputStream);

    }

    @Override
    public Object readByOss(String className) {

        String string = OssUtil.readFileContentFromOSS(bucketName, className);

        return string;
    }

    public static final String engineWorkSourcePath = "home/source/";
    public static final String engineWorkClassPath = "home/class/";
   //
    //
     public static final String className_prefix = "com.muyu.rule.server.basic.engine.value.";


//todo
    @Override
    public void HotLoadClass(String className, String versionClazz) {
        try {
        String ossFilePath = className + Suffix_JAVA;
   String ossFilePath2 = className + Suffix_CLASS;
        EngineConfig engineConfig = new EngineConfig();

        //  String filePath ="home/"+ossFilePath;
        String filePath = engineConfig.getSourcePath() + ossFilePath;
            //创建服务器项目容器的文件目录路径
            File serverFile = new File(engineWorkSourcePath, ossFilePath);
            //如果目录不存在,则创建目录
            if (!serverFile.getParentFile().exists()){
                serverFile.getParentFile().mkdirs();
            }
            File classFile = new File(engineWorkClassPath, ossFilePath2);
            //如果目录不存在,则创建目录
            if (!classFile.getParentFile().exists()){
                classFile.getParentFile().mkdirs();
            }
//下载文件到指定服务器目录下
        OssUtil.downloadFileForBucket(bucketName, ossFilePath, filePath);
//编译

        JavaCompiler javaCompiler = ToolProvider.getSystemJavaCompiler();
        //
        InputStream first = null; // 程序的输入  null 用 system.in
        OutputStream second = null; // 程序的输出 null 用 system.out
        OutputStream third = null;  // 程序的错误输出 .,null 用 system.err
        // 程序编译参数 注意 我们编译目录是我们的项目目录
        String[] strings = {"-classpath", "home/lib/", "-verbose", "-d", "home/lib/", "home/source/"+className+".java"};
log.info("规则引擎的编译参数{}[]",Arrays.toString(strings));
        // 0 表示成功， 其他表示出现了错误
        System.out.println(Arrays.toString(strings));
        int i = javaCompiler.run(first, second, third, strings);

        if (i == 0 ) {
            System.out.println("成功");
        }else {
            System.out.println("错误");
        }
        // 假设这是你的外部类文件路径
        String externalClassFilePath =
                engineWorkSourcePath +className+Suffix_CLASS;
        Path classFilePath = Paths.get(externalClassFilePath);
            String externalClassDir = externalClassFilePath.substring(0, externalClassFilePath.lastIndexOf('/'));
            URL[] urls = new URL[]{new File(externalClassDir).toURI().toURL()};

            //创建自定义类加载器
            ExternalClassLoader externalClassLoader = new ExternalClassLoader(urls);

            //加载类
            //注意类名必须是完全限定名(包括包名)
          //  Class<?> clazz = externalClassLoader.loadClassFromPath(classFilePath,className_prefix+className+Suffix_CLASS);

            //创建类的实例
            //Object instance = clazz.getDeclaredConstructor().newInstance();




        } catch (IOException e) {
            throw new RuntimeException(e);
        }


        //对class文件进行自定义类加载规则引擎
    //    Map<String, Class<?>> stringClassMap = JavaBinaryClassLoader.loadClassesByLocation(engineConfig.getServerPack(), engineConfig.getClassPath());
    }

    static Map<String, BasicEngine<DataValue>> engineMap = new ConcurrentHashMap<>();

    static {
        try {
            Class<?> aClass = Class.forName("com.muyu.rule.server.basic.engine.value.ENGINE_VALUE_VFD1000_V1");
            Class<?> aClass2 = Class.forName("com.muyu.rule.server.basic.engine.value.ENGINE_VALUE_VFD1000_V2");
            try {
                engineMap.put("ENGINE_VALUE_VFD1000_V1", (BasicEngine<DataValue>) aClass.newInstance());
                engineMap.put("ENGINE_VALUE_VFD1000_V2", (BasicEngine<DataValue>) aClass2.newInstance());
            } catch (InstantiationException e) {
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }

    }

    @Override
    public void testValue(String versionClazz, DataValue dataValue) {

        BasicEngine<DataValue> valueBasicEngine = engineMap.get(versionClazz);

        valueBasicEngine.set(dataValue);

        valueBasicEngine.execution();

    }

    /**
     * 修改激活
     *
     * @param id
     * @param open
     */
    public void updateOpen(Long id, String open) {

        UpdateWrapper<RuleEngineVersion> updateWrapper = new UpdateWrapper<>();

        updateWrapper.eq("id", id).set("open", open);

        this.update(updateWrapper);
    }
}
